%CMINTEGRATE   Compute the integral over the image (segments to scalar)
%   sum = CMINTEGRATE(segments,region)

% (C) Copyright 2002-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, August 2002.

function out = cmintegrate(in,region)

% Test
if ~isstruct(in) | ~isfield(in,'coefficients') | ~isfield(in,'offsets')
   error('Expected a segment image.')
end
coef = in.coefficients;
offset = in.offsets;
N = size(coef,2);
if N~=length(offset)-1
   error('Segment image is not correct.')
end
if nargin < 2 | isempty(region)
   region = offset([1,end]);
else
   if ~isnumeric(region) | prod(size(region))~=2
      error('REGION must have 2 elements.')
   end
   if region(1) < offset(1) | region(2) > offset(end)
      error('REGION falls (partially) outside of image.')
   end
end

% Integrate
I = 1;
x = region(1);
while offset(I+1) <= x
   I = I+1;
end
out = int(coef(:,I),[x,offset(I+1)]-offset(I));
I = I+1;
while 1
   if offset(I+1) >= region(2)
      break;
   end
   out = out + int(coef(:,I),[0,offset(I+1)-offset(I)]);
   I = I+1;
end
if offset(I) < region(2)
   out = out + int(coef(:,I),[0,region(2)-offset(I)]);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Returns the integral of the polynome over a region
function out = int(coef,region)
x = region(2);
out = [x^4/4,x^3/3,x^2/2,x]*coef;
x = region(1);
if x~=0
   out = out - [x^4/4,x^3/3,x^2/2,x]*coef;
end
