%CMLISTMAXIMA   Compute position and value of all maxima

% (C) Copyright 2002-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, August 2002.

function [mx,my] = cmlistmaxima(in)

% Test
if ~isstruct(in) | ~isfield(in,'coefficients') | ~isfield(in,'offsets')
   error('Expected a segment image.')
end
coef = in.coefficients;
offset = in.offsets;
N = size(coef,2);
if N~=length(offset)-1
   error('Segment image is not correct.')
end

A = [0,0,0,0;3,0,0,0;0,2,0,0;0,0,1,0]; % derivative of polynome
mx = [];
my = [];
for ii=1:N
   a = coef(:,ii);
   region = offset([ii,ii+1]);
   % find local extrema: solve y' = 0
   mp = [];
   if a(1)==0
      if a(2)~=0  % 1st. order polynomial
         mp = -a(3)/(2*a(2));
      end
   else           % 2nd. order polynomial
      tmp = 3*a(1);
      D = a(2)*a(2) - tmp*a(3);
      if D>0
         sqrtD = sqrt(D);
         mp = [ (-a(2)+sqrtD)/tmp ; (-a(2)-sqrtD)/tmp ];
      elseif D==0
         mp = -a(2)/tmp;
      end
   end
   % look for x out of segment
   if ~isempty(mp)
      I = mp>0 & mp<diff(region);
      mp = mp(I);
   end
   % is it (or one of the two) a maximum?: y" < 0
   for jj=1:length(mp)
      x = mp(jj);
      if (x*(6*a(1))+2*a(2)) < 0
         mx = [mx,x+region(1)];
         my = [my,cmevaluatepoly(a,x)];
      end
   end
   if ii~=N
      % if (y'(ii,end) > 0) and (y'(ii+1,0) < 0) there is a maximum in between the
      % two line segments. If any is 0, we also want it in the list...
      v1 = cmevaluatepoly(A*a,diff(region));
      %v2 = cmevaluatepoly(A*coef(:,ii+1),0);
      v2 = coef(3,ii+1);
      if v1>=0 & v2<=0
         mx = [mx,region(2)];
         %my = [my,cmevaluatepoly(coef(:,ii+1),0)];
         my = [my,coef(4,ii+1)];
      end
   end
end
